      program test_kapfxt
      implicit none
      include 'vector_eos.dek'
      include 'vector_kap.dek'

 
! tests the opacity routines
! 
! ionmax  = number of isotopes in the network
! xmass   = mass fractions
! ymass   = molar fractions
! aion    = number of nucleons
! zion    = number of protons

      integer          ionmax
      parameter        (ionmax=3)
      double precision xmass(ionmax),ymass(ionmax), &
                       aion(ionmax),zion(ionmax),wion(ionmax), &
                       temp,den,abar,zbar,wbar,ye,nxcess, &
                       pin,xin,ein,orad,ocond,opac

! a format statement
01    format(1x,a6,t10,a6,1pe10.3, &
             t30,a6,1pe10.3,t50,a6,1pe10.3)


! set the mass fractions, z's and a's of the composition
! hydrogen
      xmass(1) = 0.75d0
      aion(1)  = 1.0d0
      zion(1)  = 1.0d0
      wion(1)  = aion(1)

! helium
      xmass(2) = 0.23d0
      aion(2)  = 4.0d0
      zion(2)  = 2.0d0
      wion(2)  = aion(2)

! carbon 12
      xmass(3) = 0.02d0
      aion(3)  = 12.0d0
      zion(3)  = 6.0d0
      wion(3)  = aion(3)


! get abar, zbar and a few other composition variables
      call azbar(xmass,aion,zion,wion,ionmax, &
                 ymass,abar,zbar,wbar,ye,nxcess)

! set the input vector
      temp_row(1) = 1.0d8
      den_row(1)  = 1.0d6
      abar_row(1) = abar
      zbar_row(1) = zbar

! here the eos pipeline is only 1 element long
      jlo_eos = 1
      jhi_eos = 1


! call an equation of state
      call read_helm_table
      call helmeos


! one must call the eos before calling the opacity routine
! since some of the required input must come from an eos:

! load the electron-positron pressure and its derivatives
       pin_row(1)   = pele_row(1) + ppos_row(1)
       dpind_row(1) = dpepd_row(1)
       dpint_row(1) = dpept_row(1)


! load the electron-positron number densities and its derivatives
       xin_row(1)   = xne_row(1) + xnp_row(1)
       dxind_row(1) = dxned_row(1)
       dxint_row(1) = dxnet_row(1)


! load the electron chemical potential and its derivatives
       ein_row(1)   = etaele_row(1)
       deind_row(1) = detad_row(1)
       deint_row(1) = detat_row(1)



! get the composition variables
      call kapbar(xmass,aion,zion,ionmax, &
                  xh_row(1),xhe_row(1),xz_row(1), &
                  w4_row(1),w5_row(1),w6_row(1),zalf_row(1))


! here the opacity pipeline is only 1 element long
       jlo_kap = 1
       jhi_kap = 1


! get the opacities and conductivities
      call kapfxt



! write out what we got
      write(6,01) 'kapfxt','orad =',orad_row(1), &
                           'dord =',dord_row(1), &
                           'dort =',dort_row(1)

      write(6,01) '      ','ocond=',ocond_row(1), &
                           'docd =',docd_row(1), &
                           'doct =',doct_row(1)

      write(6,01) '      ','opac =',opac_row(1), &
                           'dopd =',dopd_row(1), &
                           'dopt =',dopt_row(1)

      write(6,*)
      write(6,01) '      ','srad =',s2rad_row(1), &
                           'dsrd =',dsrd_row(1), &
                           'dsrt =',dsrt_row(1)

      write(6,01) '      ','scond=',scond_row(1), &
                           'dscd =',dscd_row(1), &
                           'dsct =',dsct_row(1)

      write(6,01) '      ','sigma=',sigma_row(1), &
                           'dsigd=',dsigd_row(1), &
                           'dsigt=',dsigt_row(1)

      write(6,*) ' '

      stop 'normal termination'
      end  program test_kapfxt


      include 'azbar.f90'
      include 'kapbar.f90'
      include 'kapfxt.f90'
      include 'helmholtz_for_public.f90'

